==> MicroSweeper-1.0.15.tar.bz2: MenuForm.java <==
/*
 * Copyright 2008,2010 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import java.util.Vector;

public class MenuForm
    extends Form
    implements CommandListener, ItemStateListener
{
    private final Command playCommand = new Command("Play new", Command.OK, 0);
    private final Command exitCommand = new Command("Exit", Command.EXIT, 0);
    private final Command continueCommand = new Command("Continue", Command.BACK, 1);

    public interface SubgroupVisitor {
	void visit(NameBasedChoiceGroup group);
    }

    private abstract class NameBasedChoiceGroup extends ChoiceGroup {
	int _type;

	public NameBasedChoiceGroup(String title, int type) {
	    super (title, type);
	    _type = type;
	}

	public int choiceType() {
	    return _type;
	}

	public int indexForName(String name) {
	    for (int i = 0; i < size(); ++i)
		if (nameForIndex(i).equals(name))
		    return i;
	    return -1;
	}

	public void setSelectedName(String name) {
	    int idx = indexForName(name);
	    if (idx >= 0)
		setSelectedIndex(idx, true);
	}

	public String nameForIndex(int i) {
	    return getString(i).toLowerCase().replace(' ', '_');
	}

	public String getSelectedName() {
	    return nameForIndex(getSelectedIndex());
	}

	public NameBasedChoiceGroup getSubgroup(String name) {
	    return null;
	}

	public void eachSubgroup(SubgroupVisitor visitor) {
	}

	abstract public String getName();
    }

    private class OptDifficulty extends NameBasedChoiceGroup {
	OptDifficulty() {
	    super ("How Many Mines", Choice.EXCLUSIVE);
	    append("Very Few", null);
	    append("Few", null);
	    setSelectedIndex(append("Some", null), true);
	    append("Many", null);
	    append("A Lot", null);
	}

	public int howManyMines(ParticipantSelector parSel) {
	    return parSel.howManyMines(getSelectedIndex(), size());
	}

	public String getName() {
	    return "difficulty";
	}
    }

    private abstract class SelCtorBuilder {
	public abstract String name();
	public abstract ParticipantSelector build();
	public boolean isDefault() { return false; }
    }

    private abstract class OptSelector extends NameBasedChoiceGroup {
	private final SelCtorBuilder[] _ctors;
	private final String _name;
	public OptSelector(String name, SelCtorBuilder[] ctors) {
	    super("Neighborhood", Choice.EXCLUSIVE);
	    _ctors = ctors;
	    _name = name;
	    for (int i = 0; i < ctors.length; ++i) {
		int idx = append(ctors[i].name(), null);
		if (ctors[i].isDefault())
		    setSelectedIndex(idx, true);
	    }
	}

	public ParticipantSelector parSel() {
	    return _ctors[getSelectedIndex()].build();
	}

	public String getName() {
	    return _name;
	}
    }

    private class OptSelector8 extends OptSelector {
	OptSelector8(String name) {
	    super(name, new SelCtorBuilder[] {
		    new SelCtorBuilder() {
			public String name() { return "4"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel4();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "8"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel8();
			}
			public boolean isDefault() { return true; }
		    },
		    new SelCtorBuilder() {
			public String name() { return "8/4"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel84();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "12"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel12();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "16"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel16();
			}
		    }
		});
	}
    }

    private class OptSelector6 extends OptSelector {
	OptSelector6(String name) {
	    super(name, new SelCtorBuilder[] {
		    new SelCtorBuilder() {
			public String name() { return "3"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel3();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "6"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel6();
			}
			public boolean isDefault() { return true; }
		    },
		    new SelCtorBuilder() {
			public String name() { return "6/3"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel63();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "9"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel9();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "12"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel12();
			}
		    },
		});
	}
    }

    private class OptBoard extends NameBasedChoiceGroup {
	private final OptSelector8 optSelector8 = new OptSelector8("8");
	private final OptSelector6 optSelector6 = new OptSelector6("6");

	OptBoard() {
	    super("Board Type", Choice.EXCLUSIVE);
	    setSelectedIndex(append("6", null), true);
	    append("8", null);
	}

	public MineField mineField(int w, int h,
				   boolean wrapX, boolean wrapY,
				   ParticipantSelector parSel) {
	    switch (getSelectedIndex()) {
	    case 0: return new MineField6(w, h, wrapX, wrapY, parSel);
	    case 1: return new MineField8(w, h, wrapX, wrapY, parSel);
	    default: return null;
	    }
	}

	public OptSelector optSelector(int index) {
	    switch (index) {
	    case 0: return optSelector6;
	    case 1: return optSelector8;
	    default: return null;
	    }
	}

	public OptSelector optSelector() {
	    return optSelector(getSelectedIndex());
	}

	public NameBasedChoiceGroup getSubgroup(String name) {
	    return optSelector(indexForName(name));
	}

	public void eachSubgroup(SubgroupVisitor visitor) {
	    visitor.visit(optSelector6);
	    visitor.visit(optSelector8);
	}

	public String getName() {
	    return "board";
	}
    }

    private class OptConfiguration extends NameBasedChoiceGroup {
	private final boolean[] resultArray;
	private int numOptions = 0;

	private void addOption(String name, boolean on) {
	    setSelectedIndex(append(name, null), on);
	    ++numOptions;
	}

	OptConfiguration() {
	    super ("Configuration", Choice.MULTIPLE);
	    addOption("Deterministic", true);
	    addOption("Torus", false);
	    resultArray = new boolean[numOptions];
	}

	public boolean determinismEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[0];
	}

	public boolean torusEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[1];
	}

	public String getName() {
	    return "config";
	}
    }

    private final OptDifficulty optDifficulty = new OptDifficulty();
    private final OptBoard optBoard = new OptBoard();
    private final OptConfiguration optConfiguration = new OptConfiguration();
    private final NameBasedChoiceGroup[] optGroups = {
	optDifficulty, optBoard, optConfiguration
    };

    private final MicroSweeper parent;

    public MenuForm(MicroSweeper parent) {
	super("MicroSweeper");
	this.parent = parent;

	for (int i = 0; i < optGroups.length; ++i)
	    append(optGroups[i]);

	addCommand(playCommand);
	addCommand(exitCommand);

	setCommandListener(this);
	setItemStateListener(this);

	itemStateChanged(optBoard); // adds selector option
    }

    public int howManyMines() {
	return optDifficulty.howManyMines(optBoard.optSelector().parSel());
    }

    public boolean wrapX() {
	return optConfiguration.torusEnabled();
    }

    public boolean wrapY() {
	return wrapX();
    }

    public MineField mineField(int w, int h, boolean wrapX, boolean wrapY,
			       ParticipantSelector parSel) {
	return optBoard.mineField(w, h, wrapX, wrapY, parSel);
    }

    public ParticipantSelector parSel() {
	return optBoard.optSelector().parSel();
    }

    public MineFieldAcceptor acceptor() {
	return optConfiguration.determinismEnabled()
	    ? new DeterministicMineFieldAcceptor() : null;
    }

    public void commandAction(Command c, Displayable s) {
	if (c == exitCommand)
	    parent.exitAction();
	else if (c == continueCommand)
	    parent.continueGame();
	else
	    parent.playAction();
    }

    public void itemStateChanged(Item item) {
	if (item == optBoard) {
	    for (int i = 0; i < size(); ++i) {
		ChoiceGroup c = (ChoiceGroup)get(i);
		if (c instanceof OptSelector) {
		    delete(i);
		    break;
		}
	    }
	    for (int i = 0; i < size(); ++i) {
		if (get(i) == optBoard) {
		    insert(i + 1, optBoard.optSelector());
		    break;
		}
	    }
	}
    }

    private boolean gameActive = false;
    public void haveActiveGame(boolean status) {
	if (gameActive != status) {
	    if (status)
		addCommand(continueCommand);
	    else
		removeCommand(continueCommand);
	    gameActive = status;
	}
    }

    public void applySetting(Setting setting) {
	String[] key = Split.split(setting.key, ".");
	NameBasedChoiceGroup group = null;
	for (int i = 0; i < optGroups.length; ++i)
	    if (key[0].equals(optGroups[i].getName())) {
		group = optGroups[i];
		break;
	    }
	if (group == null)
	    return;

	boolean multi = false;
	for (int i = 1; i < key.length; ++i) {
	    if (group.choiceType() == Choice.MULTIPLE) {
		multi = true;
		break;
	    }
	    group = group.getSubgroup(key[i]);
	    if (group == null)
		return;
	}

	if (!multi)
	    group.setSelectedName(setting.value);
	else
	    group.setSelectedIndex(group.indexForName(key[key.length-1]),
				   setting.value.equals("true"));

	itemStateChanged(group);
    }

    private String formKey(String prefix, String suffix) {
	if (prefix.equals(""))
	    return suffix;
	else
	    return prefix + "." + suffix;
    }

    private void addSettings(String prefix, NameBasedChoiceGroup group,
			     final Vector settings) {
	final String key = formKey(prefix, group.getName());

	if (group.choiceType() == Choice.MULTIPLE) {
	    boolean[] selected = new boolean[group.size()];
	    group.getSelectedFlags(selected);
	    for (int i = 0; i < selected.length; ++i) {
		String value = selected[i] ? "true" : "false";
		String subkey = formKey(key, group.nameForIndex(i));
		Setting setting = new Setting(subkey, value);
		settings.addElement(setting);
	    }
	}
	else {
	    Setting setting = new Setting(key, group.getSelectedName());
	    settings.addElement(setting);
	}

	group.eachSubgroup(new SubgroupVisitor() {
		public void visit(NameBasedChoiceGroup group) {
		    addSettings(key, group, settings);
		}
	    });
    }

    public Vector getSettings() {
	final Vector settings = new Vector();
	for (int i = 0; i < optGroups.length; ++i)
	    addSettings("", optGroups[i], settings);
	return settings;
    }
}

==> MicroSweeper-1.0.16.tar.bz2: MenuForm.java <==
/*
 * Copyright 2008,2010,2012 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import java.util.Vector;

public class MenuForm
    extends Form
    implements CommandListener, ItemStateListener
{
    private final Command playCommand = new Command("Play new", Command.OK, 0);
    private final Command exitCommand = new Command("Exit", Command.EXIT, 0);
    private final Command continueCommand = new Command("Continue", Command.BACK, 1);

    public interface SubgroupVisitor {
	void visit(NameBasedChoiceGroup group);
    }

    private abstract class NameBasedChoiceGroup extends ChoiceGroup {
	int _type;

	public NameBasedChoiceGroup(String title, int type) {
	    super (title, type);
	    _type = type;
	}

	public int choiceType() {
	    return _type;
	}

	public int indexForName(String name) {
	    for (int i = 0; i < size(); ++i)
		if (nameForIndex(i).equals(name))
		    return i;
	    return -1;
	}

	public void setSelectedName(String name) {
	    int idx = indexForName(name);
	    if (idx >= 0)
		setSelectedIndex(idx, true);
	}

	public String nameForIndex(int i) {
	    return getString(i).toLowerCase().replace(' ', '_');
	}

	public String getSelectedName() {
	    return nameForIndex(getSelectedIndex());
	}

	public NameBasedChoiceGroup getSubgroup(String name) {
	    return null;
	}

	public void eachSubgroup(SubgroupVisitor visitor) {
	}

	abstract public String getName();
    }

    private class OptDifficulty extends NameBasedChoiceGroup {
	OptDifficulty() {
	    super ("How Many Mines", Choice.EXCLUSIVE);
	    append("Very Few", null);
	    append("Few", null);
	    setSelectedIndex(append("Some", null), true);
	    append("Many", null);
	    append("A Lot", null);
	}

	public int howManyMines(ParticipantSelector parSel) {
	    return parSel.howManyMines(getSelectedIndex(), size());
	}

	public String getName() {
	    return "difficulty";
	}
    }

    private abstract class SelCtorBuilder {
	public abstract String name();
	public abstract ParticipantSelector build();
	public boolean isDefault() { return false; }
    }

    private abstract class OptSelector extends NameBasedChoiceGroup {
	private final SelCtorBuilder[] _ctors;
	private final String _name;
	public OptSelector(String name, SelCtorBuilder[] ctors) {
	    super("Neighborhood", Choice.EXCLUSIVE);
	    _ctors = ctors;
	    _name = name;
	    for (int i = 0; i < ctors.length; ++i) {
		int idx = append(ctors[i].name(), null);
		if (ctors[i].isDefault())
		    setSelectedIndex(idx, true);
	    }
	}

	public ParticipantSelector parSel() {
	    return _ctors[getSelectedIndex()].build();
	}

	public String getName() {
	    return _name;
	}
    }

    private class OptSelector8 extends OptSelector {
	OptSelector8(String name) {
	    super(name, new SelCtorBuilder[] {
		    new SelCtorBuilder() {
			public String name() { return "4"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel4();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "8"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel8();
			}
			public boolean isDefault() { return true; }
		    },
		    new SelCtorBuilder() {
			public String name() { return "8/4"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel84();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "12"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel12();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "16"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel16();
			}
		    }
		});
	}
    }

    private class OptSelector6 extends OptSelector {
	OptSelector6(String name) {
	    super(name, new SelCtorBuilder[] {
		    new SelCtorBuilder() {
			public String name() { return "3"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel3();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "6"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel6();
			}
			public boolean isDefault() { return true; }
		    },
		    new SelCtorBuilder() {
			public String name() { return "6/3"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel63();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "9"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel9();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "12"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel12();
			}
		    },
		});
	}
    }

    private class OptBoard extends NameBasedChoiceGroup {
	private final OptSelector8 optSelector8 = new OptSelector8("8");
	private final OptSelector6 optSelector6 = new OptSelector6("6");

	OptBoard() {
	    super("Board Type", Choice.EXCLUSIVE);
	    setSelectedIndex(append("6", null), true);
	    append("8", null);
	}

	public MineField mineField(int w, int h,
				   boolean wrapX, boolean wrapY,
				   ParticipantSelector parSel,
				   boolean sums) {
	    switch (getSelectedIndex()) {
	    case 0: return new MineField6(w, h, wrapX, wrapY, parSel, sums);
	    case 1: return new MineField8(w, h, wrapX, wrapY, parSel, sums);
	    default: return null;
	    }
	}

	public OptSelector optSelector(int index) {
	    switch (index) {
	    case 0: return optSelector6;
	    case 1: return optSelector8;
	    default: return null;
	    }
	}

	public OptSelector optSelector() {
	    return optSelector(getSelectedIndex());
	}

	public NameBasedChoiceGroup getSubgroup(String name) {
	    return optSelector(indexForName(name));
	}

	public void eachSubgroup(SubgroupVisitor visitor) {
	    visitor.visit(optSelector6);
	    visitor.visit(optSelector8);
	}

	public String getName() {
	    return "board";
	}
    }

    private class OptConfiguration extends NameBasedChoiceGroup {
	private final boolean[] resultArray;
	private int numOptions = 0;

	private void addOption(String name, boolean on) {
	    setSelectedIndex(append(name, null), on);
	    ++numOptions;
	}

	OptConfiguration() {
	    super ("Configuration", Choice.MULTIPLE);
	    addOption("Deterministic", true);
	    addOption("Torus", false);
	    addOption("Row/Col Sums", false);
	    addOption("Full Plan", false);
	    resultArray = new boolean[numOptions];
	}

	public boolean determinismEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[0];
	}

	public boolean torusEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[1];
	}

	public boolean sumsEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[2];
	}

	public boolean fullPlan() {
	    getSelectedFlags(resultArray);
	    return resultArray[3];
	}

	public String getName() {
	    return "config";
	}
    }

    private final OptDifficulty optDifficulty = new OptDifficulty();
    private final OptBoard optBoard = new OptBoard();
    private final OptConfiguration optConfiguration = new OptConfiguration();
    private final NameBasedChoiceGroup[] optGroups = {
	optDifficulty, optBoard, optConfiguration
    };

    private final MicroSweeper parent;

    public MenuForm(MicroSweeper parent) {
	super("MicroSweeper");
	this.parent = parent;

	for (int i = 0; i < optGroups.length; ++i)
	    append(optGroups[i]);

	addCommand(playCommand);
	addCommand(exitCommand);

	setCommandListener(this);
	setItemStateListener(this);

	itemStateChanged(optBoard); // adds selector option
    }

    public int howManyMines() {
	return optDifficulty.howManyMines(optBoard.optSelector().parSel());
    }

    public boolean wrapX() {
	return optConfiguration.torusEnabled();
    }

    public boolean wrapY() {
	return wrapX();
    }

    public MineField mineField(int w, int h, boolean wrapX, boolean wrapY,
			       ParticipantSelector parSel, boolean sums) {
	return optBoard.mineField(w, h, wrapX, wrapY, parSel, sums);
    }

    public ParticipantSelector parSel() {
	return optBoard.optSelector().parSel();
    }

    public boolean sumsEnabled() {
	return optConfiguration.sumsEnabled();
    }

    public boolean fullPlan() {
	return optConfiguration.fullPlan();
    }

    public MineFieldAcceptor acceptor() {
	return optConfiguration.determinismEnabled()
	    ? new DeterministicMineFieldAcceptor() : null;
    }

    public void commandAction(Command c, Displayable s) {
	if (c == exitCommand)
	    parent.exitAction();
	else if (c == continueCommand)
	    parent.continueGame();
	else if (c == playCommand)
	    parent.playAction();
    }

    public void itemStateChanged(Item item) {
	if (item == optBoard) {
	    for (int i = 0; i < size(); ++i) {
		ChoiceGroup c = (ChoiceGroup)get(i);
		if (c instanceof OptSelector) {
		    delete(i);
		    break;
		}
	    }
	    for (int i = 0; i < size(); ++i) {
		if (get(i) == optBoard) {
		    insert(i + 1, optBoard.optSelector());
		    break;
		}
	    }
	}
    }

    private boolean gameActive = false;
    public void haveActiveGame(boolean status) {
	if (gameActive != status) {
	    if (status)
		addCommand(continueCommand);
	    else
		removeCommand(continueCommand);
	    gameActive = status;
	}
    }

    public void applySetting(Setting setting) {
	String[] key = Split.split(setting.key, ".");
	NameBasedChoiceGroup group = null;
	for (int i = 0; i < optGroups.length; ++i)
	    if (key[0].equals(optGroups[i].getName())) {
		group = optGroups[i];
		break;
	    }
	if (group == null)
	    return;

	boolean multi = false;
	for (int i = 1; i < key.length; ++i) {
	    if (group.choiceType() == Choice.MULTIPLE) {
		multi = true;
		break;
	    }
	    group = group.getSubgroup(key[i]);
	    if (group == null)
		return;
	}

	if (!multi)
	    group.setSelectedName(setting.value);
	else {
	    int idx = group.indexForName(key[key.length-1]);
	    if (idx > 0)
		group.setSelectedIndex(idx, setting.value.equals("true"));
	}

	itemStateChanged(group);
    }

    private String formKey(String prefix, String suffix) {
	if (prefix.equals(""))
	    return suffix;
	else
	    return prefix + "." + suffix;
    }

    private void addSettings(String prefix, NameBasedChoiceGroup group,
			     final Vector settings) {
	final String key = formKey(prefix, group.getName());

	if (group.choiceType() == Choice.MULTIPLE) {
	    boolean[] selected = new boolean[group.size()];
	    group.getSelectedFlags(selected);
	    for (int i = 0; i < selected.length; ++i) {
		String value = selected[i] ? "true" : "false";
		String subkey = formKey(key, group.nameForIndex(i));
		Setting setting = new Setting(subkey, value);
		settings.addElement(setting);
	    }
	}
	else {
	    Setting setting = new Setting(key, group.getSelectedName());
	    settings.addElement(setting);
	}

	group.eachSubgroup(new SubgroupVisitor() {
		public void visit(NameBasedChoiceGroup group) {
		    addSettings(key, group, settings);
		}
	    });
    }

    public Vector getSettings() {
	final Vector settings = new Vector();
	for (int i = 0; i < optGroups.length; ++i)
	    addSettings("", optGroups[i], settings);
	return settings;
    }
}
